/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 * $Id: CimiManagerAbstract.java 1168 2012-04-26 17:21:54Z antonma $
 *
 */
package org.ow2.sirocco.apis.rest.cimi.manager;

import javax.ws.rs.core.Response;

import org.ow2.sirocco.apis.rest.cimi.converter.InvalidConversionException;
import org.ow2.sirocco.apis.rest.cimi.domain.CimiEntityType;
import org.ow2.sirocco.apis.rest.cimi.domain.CimiJob;
import org.ow2.sirocco.apis.rest.cimi.request.CimiRequest;
import org.ow2.sirocco.apis.rest.cimi.request.CimiResponse;
import org.ow2.sirocco.apis.rest.cimi.utils.Constants;
import org.ow2.sirocco.cloudmanager.core.api.exception.CloudProviderException;
import org.ow2.sirocco.cloudmanager.core.api.exception.InvalidRequestException;
import org.ow2.sirocco.cloudmanager.core.api.exception.ResourceConflictException;
import org.ow2.sirocco.cloudmanager.core.api.exception.ResourceNotFoundException;
import org.ow2.sirocco.cloudmanager.core.api.exception.ServiceUnavailableException;
import org.ow2.sirocco.cloudmanager.model.cimi.Job;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Abstract class to manage the phases of validation, conversion and calling
 * services of requests.
 */
public abstract class CimiManagerAbstract implements CimiManager {

    /** Logger */
    private static final Logger LOGGER = LoggerFactory.getLogger(CimiManagerAbstract.class);

    /**
     * {@inheritDoc}
     * <p>
     * In case of catch exception, the state is directly updated in response as
     * shown in the table below:
     * <table>
     * <tr>
     * <th>Exception</th>
     * <th>HTTP status code</th>
     * </tr>
     * <tr>
     * <td>ResourceNotFoundException</td>
     * <td>404</td>
     * </tr>
     * <tr>
     * <td>InvalidConversionException</td>
     * <td>400</td>
     * </tr>
     * <tr>
     * <td>InvalidRequestException</td>
     * <td>400</td>
     * </tr>
     * <tr>
     * <td>ResourceConflictException</td>
     * <td>409</td>
     * </tr>
     * <tr>
     * <td>ServiceUnavailableException</td>
     * <td>503</td>
     * </tr>
     * <tr>
     * <td>SecurityException</td>
     * <td>403</td>
     * </tr>
     * <tr>
     * <td>UnsupportedOperationException</td>
     * <td>501</td>
     * </tr>
     * <tr>
     * <td>CloudProviderException</td>
     * <td>500</td>
     * </tr>
     * <tr>
     * <td>Exception</td>
     * <td>500</td>
     * </tr>
     * </table>
     * </p>
     * 
     * @see org.ow2.sirocco.apis.rest.cimi.manager.CimiManager#execute(org.ow2.sirocco.apis.rest.cimi.request.CimiRequest,
     *      org.ow2.sirocco.apis.rest.cimi.request.CimiResponse)
     */
    @Override
    public void execute(final CimiRequest request, final CimiResponse response) {
        try {
            if (this.doValidate(request, response)) {
                Object dataServiceIn = this.doConvertToDataService(request, response);
                if (false == response.isCommitted()) {
                    Object dataServiceOut = this.doCallService(request, response, dataServiceIn);
                    if (false == response.isCommitted()) {
                        this.doConvertToResponse(request, response, dataServiceOut);
                    }
                }
            }
        } catch (InvalidConversionException e) {
            this.convertToResponse(request, response, e);
        } catch (ResourceNotFoundException e) {
            this.convertToResponse(request, response, e);
        } catch (InvalidRequestException e) {
            this.convertToResponse(request, response, e);
        } catch (ResourceConflictException e) {
            this.convertToResponse(request, response, e);
        } catch (ServiceUnavailableException e) {
            this.convertToResponse(request, response, e);
        } catch (SecurityException e) {
            this.convertToResponse(request, response, e);
        } catch (UnsupportedOperationException e) {
            this.convertToResponse(request, response, e);
        } catch (CloudProviderException e) {
            this.convertToResponse(request, response, e);
        } catch (Exception e) {
            this.convertToResponse(request, response, e);
        }
    }

    /**
     * Validate the request.
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @return True if the request is valid
     * @throws Exception In case of validation error
     */
    protected abstract boolean validate(CimiRequest request, CimiResponse response) throws Exception;

    /**
     * Convert the CIMI data from request to a service data.
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @return The input service data
     * @throws Exception In case of conversion error
     */
    protected abstract Object convertToDataService(CimiRequest request, CimiResponse response) throws Exception;

    /**
     * Call the service.
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param dataService The input service data
     * @return The output service data or null if none output
     * @throws Exception In case of error in service
     */
    protected abstract Object callService(CimiRequest request, CimiResponse response, Object dataService) throws Exception;

    /**
     * Convert the service data to a CIMI data from request.
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param dataService The output service data
     * @throws Exception In case of conversion error
     */
    protected abstract void convertToResponse(CimiRequest request, CimiResponse response, Object dataService) throws Exception;

    /**
     * Call after the conversion.
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param dataService The output service data
     */
    protected void afterConvertToResponse(final CimiRequest request, final CimiResponse response, final Object dataService) {
        if (null == response.getCimiData()) {
            // Job
            if (dataService instanceof Job) {
                CimiJob cimi = (CimiJob) request.getContext().getRootConverter(CimiEntityType.Job)
                    .toCimi(request.getContext(), dataService);
                response.setCimiData(cimi);
                response.putHeader(Constants.HEADER_CIMI_JOB_URI, cimi.getId());
                cimi.setTargetEntity(request.getContext().makeHref(request.getCimiData(), cimi.getTargetEntity()));
                response.putHeader(Constants.HEADER_LOCATION, cimi.getTargetEntity());
                response.setStatus(Response.Status.ACCEPTED);
            }
        }
    }

    /**
     * Manage the request validation.
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @return True if the request is valid
     * @throws Exception In case of validation error
     */
    private boolean doValidate(final CimiRequest request, final CimiResponse response) throws Exception {
        boolean valid = false;
        valid = this.validate(request, response);
        if (!valid) {
            response.setStatus(Response.Status.BAD_REQUEST);
        }
        return valid;
    }

    /**
     * Manage the conversion of the CIMI data from request to a service data.
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @return The input service data
     * @throws Exception In case of conversion error
     */
    private Object doConvertToDataService(final CimiRequest request, final CimiResponse response) throws Exception {
        Object dataService = null;
        dataService = this.convertToDataService(request, response);
        return dataService;
    }

    /**
     * Manage the call to the service.
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param dataServiceIn The input service data
     * @return The output service data or null if none output
     * @throws Exception In case of service error
     */
    private Object doCallService(final CimiRequest request, final CimiResponse response, final Object dataServiceIn)
        throws Exception {
        Object dataServiceOut = null;
        dataServiceOut = this.callService(request, response, dataServiceIn);
        return dataServiceOut;
    }

    /**
     * Manage the conversion of a service data to a CIMI data from request.
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param dataService The output service data
     * @throws Exception In case of conversion error
     */
    private void doConvertToResponse(final CimiRequest request, final CimiResponse response, final Object dataService)
        throws Exception {
        this.convertToResponse(request, response, dataService);
        this.afterConvertToResponse(request, response, dataService);
    }

    /**
     * Convert general exception to HTTP status "INTERNAL_SERVER_ERROR" (500).
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param exception The exception to convert
     */
    private void convertToResponse(final CimiRequest request, final CimiResponse response, final Exception exception) {
        CimiManagerAbstract.LOGGER.error("Internal Server Error", exception);
        response.setStatus(Response.Status.INTERNAL_SERVER_ERROR);
        response.setErrorMessage(exception.getMessage());
    }

    /**
     * Convert exception to HTTP status "NOT FOUND" (404).
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param exception The exception to convert
     */
    private void convertToResponse(final CimiRequest request, final CimiResponse response,
        final ResourceNotFoundException exception) {
        CimiManagerAbstract.LOGGER.debug("Resource not found : {}", request.getId());
        response.setStatus(Response.Status.NOT_FOUND);
        response.setErrorMessage(exception.getMessage());
    }

    /**
     * Convert exception to HTTP status "BAD REQUEST" (400).
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param exception The exception to convert
     */
    private void convertToResponse(final CimiRequest request, final CimiResponse response,
        final InvalidRequestException exception) {
        CimiManagerAbstract.LOGGER.debug(exception.getMessage(), exception);
        response.setStatus(Response.Status.BAD_REQUEST);
        response.setErrorMessage(exception.getMessage());
    }

    /**
     * Convert exception to HTTP status "BAD REQUEST" (400).
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param exception The exception to convert
     */
    private void convertToResponse(final CimiRequest request, final CimiResponse response,
        final InvalidConversionException exception) {
        CimiManagerAbstract.LOGGER.debug(exception.getMessage(), exception);
        response.setStatus(Response.Status.BAD_REQUEST);
        response.setErrorMessage(exception.getMessage());
    }

    /**
     * Convert exception to HTTP status "CONFLICT" (409).
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param exception The exception to convert
     */
    private void convertToResponse(final CimiRequest request, final CimiResponse response,
        final ResourceConflictException exception) {
        CimiManagerAbstract.LOGGER.debug(exception.getMessage(), exception);
        response.setStatus(Response.Status.CONFLICT);
        response.setErrorMessage(exception.getMessage());
    }

    /**
     * Convert exception to HTTP status "SERVICE UNAVAILABLE" (503).
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param exception The exception to convert
     */
    private void convertToResponse(final CimiRequest request, final CimiResponse response,
        final ServiceUnavailableException exception) {
        CimiManagerAbstract.LOGGER.debug(exception.getMessage(), exception);
        response.setStatus(Response.Status.SERVICE_UNAVAILABLE);
        response.setErrorMessage(exception.getMessage());
    }

    /**
     * Convert exception to HTTP status "FORBIDDEN" (403).
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param exception The exception to convert
     */
    private void convertToResponse(final CimiRequest request, final CimiResponse response, final SecurityException exception) {
        CimiManagerAbstract.LOGGER.debug(exception.getMessage(), exception);
        response.setStatus(Response.Status.FORBIDDEN);
        response.setErrorMessage(exception.getMessage());
    }

    /**
     * Convert exception to HTTP status "NOT IMPLEMENTED" (501).
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param exception The exception to convert
     */
    private void convertToResponse(final CimiRequest request, final CimiResponse response,
        final UnsupportedOperationException exception) {
        CimiManagerAbstract.LOGGER.debug(exception.getMessage(), exception);
        response.setStatus(501);
        response.setErrorMessage(exception.getMessage());
    }

    /**
     * Convert exception to HTTP status "INTERNAL_SERVER_ERROR" (500).
     * 
     * @param request The CIMI request
     * @param response The CIMI response
     * @param exception The exception to convert
     */
    private void convertToResponse(final CimiRequest request, final CimiResponse response,
        final CloudProviderException exception) {
        CimiManagerAbstract.LOGGER.error(exception.getMessage(), exception);
        response.setStatus(Response.Status.INTERNAL_SERVER_ERROR);
        response.setErrorMessage(exception.getMessage());
    }

}
